import React from 'react';
import { render, fireEvent, screen, waitFor } from '@/test-utils/index';
import NsDropdown from './NsDropdown';
import NsDropdownOption from './NsDropdownOption';

function createDropdownOptions(options: { value: string; label: string }[]) {
    return options.map((option) => (
        <NsDropdownOption key={option.value} value={option.value}>
            {option.label}
        </NsDropdownOption>
    ));
}

describe('NsDropdown', () => {
    const onSelect = jest.fn();
    const selectedLabel = '';

    const options = [
        { value: 'Option 1', label: 'Option 1' },
        { value: 'Option 2', label: 'Option 2' },
        { value: 'Option 3', label: 'Option 3' },
    ];

    it('1. Should render the placeholder text when no value is selected', () => {
        render(
            <NsDropdown placeholder="Select an option" selectedLabel={selectedLabel} onSelect={onSelect}>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        expect(screen.getByText('Select an option')).toBeInTheDocument();
    });

    it('2. Should render the selected value label', () => {
        render(
            <NsDropdown selectedLabel={selectedLabel} onSelect={onSelect}>
                {createDropdownOptions(options)}
            </NsDropdown>
        );
        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);
        const dropdownOptionContainer = screen.getByTestId('dropdown-option-container-0');

        fireEvent.click(dropdownOptionContainer);
        expect(onSelect).toHaveBeenCalledWith('Option 1');
    });

    it('3. Should open and close Dropdown on click', () => {
        render(
            <NsDropdown selectedLabel={selectedLabel} onSelect={onSelect}>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        expect(screen.getByTestId('option-Option 1')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 2')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 3')).toBeInTheDocument();

        fireEvent.click(dropdownHead);
        expect(screen.queryByTestId('option-Option 1')).not.toBeInTheDocument();
        expect(screen.queryByTestId('option-Option 2')).not.toBeInTheDocument();
        expect(screen.queryByTestId('option-Option 3')).not.toBeInTheDocument();
    });

    it('4. Should close dropdown on mouse leave when closeOnMouseLeave is set to true', async () => {
        render(
            <NsDropdown closeOnMouseLeave selectedLabel={selectedLabel} onSelect={onSelect}>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        const dropdownList = screen.getByTestId('dropdown-list');
        fireEvent.mouseLeave(dropdownList);

        await waitFor(() => {
            expect(screen.queryByTestId('option-Option 1')).not.toBeInTheDocument();
            expect(screen.queryByTestId('option-Option 2')).not.toBeInTheDocument();
            expect(screen.queryByTestId('option-Option 3')).not.toBeInTheDocument();
        });
    });
    it('5. Should not close dropdown on mouse leave when closeOnMouseLeave is set to false', () => {
        render(
            <NsDropdown selectedLabel={selectedLabel} onSelect={onSelect} closeOnMouseLeave={false}>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        const dropdownList = screen.getByTestId('dropdown-list');
        fireEvent.mouseLeave(dropdownList);

        expect(screen.getByTestId('option-Option 1')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 2')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 3')).toBeInTheDocument();
    });
    it('6. Should not open dropdown when disabled is set to true', () => {
        render(
            <NsDropdown selectedLabel={selectedLabel} onSelect={onSelect} disabled>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        expect(screen.queryByTestId('dropdown-list')).not.toBeInTheDocument();
    });
    it('7. Should apply no border style when type is not outlined', () => {
        render(
            <NsDropdown selectedLabel={selectedLabel} onSelect={onSelect} type="unfilled">
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        expect(dropdownHead).toHaveStyle('border: none');
    });
});
