import { ColorUtils, FoundationTheme, styled } from '@nuskin/foundation-theme';

import { NsTypographyProps } from './types.d';

const { paletteColor } = ColorUtils;

type TypographyConfig = {
    fontFamily: string;
    fontSize?: number;
    lineHeight?: number;
    letterSpacingPercentage?: number;
    paragraphSpacing?: number;
    fontWeight?: number;
};

const defaultFont = 'Roboto';
const defaultTypographyStyles = {
    h1: {
        fontFamily: defaultFont,
        fontSize: 42,
        lineHeight: 48,
        letterSpacingPercentage: 0,
        paragraphSpacing: 0,
    },
    h2: {
        fontFamily: defaultFont,
        fontSize: 36,
        lineHeight: 42,
        letterSpacingPercentage: 0,
        paragraphSpacing: 0,
    },
    h3: {
        fontFamily: defaultFont,
        fontSize: 32,
        lineHeight: 38,
        letterSpacingPercentage: 0,
        paragraphSpacing: 0,
    },
    h4: {
        fontFamily: defaultFont,
        fontSize: 26,
        lineHeight: 34,
        letterSpacingPercentage: 0,
        paragraphSpacing: 0,
    },
    h5: {
        fontFamily: defaultFont,
        fontSize: 22,
        lineHeight: 28,
        letterSpacingPercentage: 0,
        fontWeight: 700,
        paragraphSpacing: 0,
    },
    h6: {
        fontFamily: defaultFont,
        fontSize: 18,
        lineHeight: 24,
        letterSpacingPercentage: 0,
        fontWeight: 600,
        paragraphSpacing: 0,
    },
    'title-l': {
        fontFamily: defaultFont,
        fontSize: 22,
        lineHeight: 28,
        letterSpacingPercentage: 2,
        fontWeight: 700,
        paragraphSpacing: 0,
    },
    'title-m': {
        fontFamily: defaultFont,
        fontSize: 16,
        lineHeight: 20,
        letterSpacingPercentage: 4,
        fontWeight: 700,
        paragraphSpacing: 0,
    },
    'title-s': {
        fontFamily: defaultFont,
        fontSize: 16,
        lineHeight: 16,
        letterSpacingPercentage: 4,
        fontWeight: 700,
        paragraphSpacing: 0,
    },
    'label-l': {
        fontFamily: defaultFont,
        fontSize: 20,
        lineHeight: 26,
        letterSpacingPercentage: 0,
        fontWeight: 600,
        paragraphSpacing: 8,
    },
    'label-m': {
        fontFamily: defaultFont,
        fontSize: 16,
        lineHeight: 22,
        letterSpacingPercentage: 0,
        fontWeight: 600,
        paragraphSpacing: 8,
    },
    'label-s': {
        fontFamily: defaultFont,
        fontSize: 14,
        lineHeight: 20,
        letterSpacingPercentage: 0,
        fontWeight: 700,
        paragraphSpacing: 6,
    },
    'body-xl': {
        fontFamily: defaultFont,
        fontSize: 18,
        lineHeight: 22,
        letterSpacingPercentage: 0,
        paragraphSpacing: 22,
    },
    'body-l': {
        fontFamily: defaultFont,
        fontSize: 16,
        lineHeight: 22,
        letterSpacingPercentage: 0,
        paragraphSpacing: 20,
    },
    'body-m': {
        fontFamily: defaultFont,
        fontSize: 14,
        lineHeight: 20,
        letterSpacingPercentage: 0,
        paragraphSpacing: 18,
    },
    'body-s': {
        fontFamily: defaultFont,
        fontSize: 12,
        lineHeight: 16,
        letterSpacingPercentage: 1,
        paragraphSpacing: 16,
    },
    'body-xs': {
        fontFamily: defaultFont,
        fontSize: 10,
        lineHeight: 14,
        letterSpacingPercentage: 2,
        paragraphSpacing: 14,
    },
    CTA: {
        fontFamily: defaultFont,
        fontSize: 14,
        lineHeight: 20,
        letterSpacingPercentage: 0.14,
        fontWeight: 600,
        paragraphSpacing: 0,
    },
};

const fontWeightMap = {
    light: 300,
    normal: 400,
    bold: 600,
};

const getVariantStyle = (
    themeTypographyVariants: Record<FoundationTheme.TypographyVariants, TypographyConfig>,
    variant: FoundationTheme.TypographyVariants
) => {
    const variantStyles = themeTypographyVariants?.[variant as FoundationTheme.TypographyVariants];
    if (variantStyles) {
        return variantStyles;
    }
    return defaultTypographyStyles[variant as FoundationTheme.TypographyVariants];
};

export const StyledTypography = styled('div')<NsTypographyProps>(
    ({ theme, weight, variant, noSpacing, decoration, color, colorOverride }) => {
        const fontStyles: TypographyConfig = getVariantStyle(theme?.fonts?.typographyVariants, variant) || {};
        if (noSpacing) {
            fontStyles.paragraphSpacing = 0;
        }
        if (weight) {
            fontStyles.fontWeight = fontWeightMap[weight] || fontWeightMap.normal;
        }
        const updatedFontStyle = {
            fontFamily: fontStyles?.fontFamily || defaultFont,
            fontSize: `${fontStyles?.fontSize || 16}px`,
            lineHeight: `${fontStyles?.lineHeight || 16}px`,
            margin: fontStyles?.paragraphSpacing || 0,
            fontWeight: fontStyles?.fontWeight || 400,
            letterSpacing: fontStyles?.letterSpacingPercentage || 0,
        };
        return {
            ...updatedFontStyle,
            textDecorationLine: decoration,
            color: colorOverride?.startsWith('#') ? colorOverride : paletteColor(color)({ theme }),
        };
    }
);
